----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:40:24 02/14/2022 
-- Design Name: 
-- Module Name:    ctrl - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity ctrl is
    Port ( conf_in : in  STD_LOGIC_VECTOR (15 downto 0);
           clk_in : in  STD_LOGIC;
			  clk_record_in : in  STD_LOGIC;
           done_dsp : in  STD_LOGIC;
			  done_detect : in  STD_LOGIC;
			  dbg_out : out  STD_LOGIC_VECTOR (3 downto 0);
           sign_len : out  STD_LOGIC_VECTOR (1 downto 0);
           detect_reset : out  STD_LOGIC;
           sram_oe : out  STD_LOGIC;
           sram_rw : out  STD_LOGIC;
           samples_addr : out  STD_LOGIC_VECTOR (18 downto 0);
           dsp_reset : out  STD_LOGIC;
           addr_sel : out  STD_LOGIC_VECTOR (1 downto 0);
           sign_in_sel : out  STD_LOGIC;
           adc_clk : out  STD_LOGIC;
			  ctrl_int : out  STD_LOGIC);
end ctrl;

architecture Behavioral of ctrl is

	signal adc_cnt: unsigned (18 downto 0);
	signal adc_rec_steps: unsigned (2 downto 0);
	signal sign_conv_steps: unsigned (2 downto 0);
	signal samples_max: unsigned (19 downto 0);
	signal done_record_adc: std_logic;
	signal reset_adc_rec: std_logic;
	signal sign_len_sig: std_logic_vector (1 downto 0);
	signal last_conf_in2: std_logic;
	signal reset_ctrl: std_logic;
	
begin

with sign_len_sig select
	samples_max <= "01111111011111100001" when "11",
						"00001100101111110001" when "10",
						"00000001010001011001" when "01",
						"00000000000111111101" when others;

process_start: process (clk_in) is
begin
	if (clk_in'event and clk_in = '1') then
		last_conf_in2 <= conf_in(2);
		if (conf_in(2) = '1' and last_conf_in2 = '0') then
			reset_ctrl <= '1';
		else
			reset_ctrl <= '0';
		end if;
	end if;	
end process process_start;

process_record_adc: process (clk_record_in) is
begin
	-- all in sync with 100MHz clock
	if (clk_record_in'event and clk_record_in = '1') then
		-- reset
		if (reset_adc_rec = '1') then
			adc_cnt <= (others => '0');
			samples_addr <= (others => '0');
			adc_rec_steps <= (others => '0');
			sram_oe <= '1';
			sram_rw <= '1';
			adc_clk <= '0';
			done_record_adc <= '0';
		-- record adc samples to memory
		elsif (adc_cnt /= samples_max) then
			if(adc_rec_steps = 0) then
				samples_addr <= std_logic_vector(adc_cnt);
				adc_rec_steps <= adc_rec_steps + 1;
				adc_clk <= '1';
			elsif(adc_rec_steps = 1) then
				sram_rw <= '0';
				adc_rec_steps <= adc_rec_steps + 1;
			elsif(adc_rec_steps = 2) then
				adc_rec_steps <= adc_rec_steps + 1;
			elsif(adc_rec_steps = 3) then
				sram_rw <= '1';
				adc_rec_steps <= adc_rec_steps + 1;
				adc_clk <= '0';
			elsif(adc_rec_steps = 4) then
				adc_cnt <= adc_cnt + 1;
				adc_rec_steps <= (others => '0');
			end if;
			done_record_adc <= '0';	--needed!?
		else
			done_record_adc <= '1';
			sram_oe <= '0';
			sram_rw <= '1';
			samples_addr <= (others => '0');
		end if;
	end if;
end process process_record_adc;



process_sign_conv_seq: process (clk_in) is
begin
	-- all in sync with 100MHz clock
	if (clk_in'event and clk_in = '1') then
		-- reset
		if (reset_ctrl = '1') then
			sign_conv_steps <= (others => '0');
			reset_adc_rec <= '1';
			dsp_reset <= '1';
			detect_reset <= '1';
			ctrl_int <= '0';
		elsif (sign_conv_steps = 0) then	--more time to reset everything
			sign_conv_steps <= "001";	
		elsif (sign_conv_steps = 1) then
			sign_in_sel <= '0';
			addr_sel <= "00";
			reset_adc_rec <= '0';
			if (done_record_adc = '1') then
				sign_conv_steps <= "010";
			end if;
		elsif (sign_conv_steps = 2) then
			sign_in_sel <= '1';
			addr_sel <= "01";
			dsp_reset <= '0';
			if (done_dsp = '1') then
				sign_conv_steps <= "011";
			end if;
		elsif (sign_conv_steps = 3) then
			sign_in_sel <= '1';
			addr_sel <= "10";
			detect_reset <= '0';
			if (done_detect = '1') then
				sign_conv_steps <= "100";
				ctrl_int <= '1';	-- edge to generate interrupt in microcontroller
			end if;
		end if;
	end if;
end process process_sign_conv_seq;

sign_len_sig <= conf_in(1 downto 0);
sign_len <= sign_len_sig;

dbg_out(0) <= done_record_adc;
dbg_out(1) <= done_dsp;
dbg_out(2) <= done_detect;
dbg_out(3) <= reset_ctrl;


end Behavioral;

