----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    21:29:52 04/08/2022 
-- Design Name: 
-- Module Name:    graphics - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity graphics is
    Port ( clk_in : in  STD_LOGIC;
           graph_uc_data : in  STD_LOGIC_VECTOR (9 downto 0);
           graph_uc_clk : in  STD_LOGIC;
           graph_sram_addr : out  STD_LOGIC_VECTOR (17 downto 0);
           graph_sram_cs : out  STD_LOGIC;
           graph_sram_lb : out  STD_LOGIC;
           graph_sram_ub : out  STD_LOGIC;
           graph_sram_oe : out  STD_LOGIC;
           graph_sram_we : out  STD_LOGIC;
           graph_sram_data_rd : in  STD_LOGIC_VECTOR(15 downto 0);  
           graph_sram_data_wr : out  STD_LOGIC_VECTOR(15 downto 0);
           graph_sram_data_rw : out  STD_LOGIC_VECTOR(15 downto 0);
           tft_data : out  STD_LOGIC_VECTOR (8 downto 0);
           tft_vs : out  STD_LOGIC;
           tft_hs : out  STD_LOGIC;
           tft_de : out  STD_LOGIC;
           tft_clk : out  STD_LOGIC);
end graphics;

architecture Behavioral of graphics is
	signal mem_rw_cnt : unsigned (3 downto 0) := (others => '0');
	signal tft_clk_sig : std_logic := '0';
	signal tft_data_buf : std_logic_vector(15 downto 0);
	signal graph_sram_addr_hor : unsigned (8 downto 0);
	signal tft_hcnt : unsigned (9 downto 0) := (others => '0');
	signal graph_sram_addr_vert : unsigned (8 downto 0);
	signal tft_vcnt : unsigned (8 downto 0) := (others => '0');
	signal pix_color_buf : std_logic_vector (15 downto 0);
	signal pix_addr_buf : std_logic_vector (18 downto 0);
	signal pix_rect_addr_buf : std_logic_vector (18 downto 0);
	signal pix_loaded : std_logic;
	signal pix_rect_loaded : std_logic;
	signal pix_shreg : std_logic_vector(29 downto 0);
	signal pix_shreg_clr : std_logic;	--signal to clear shift register from other process
	signal rect_cnt_x : unsigned (10 downto 0);
	signal draw_rect : std_logic;
	signal delay_cnt : unsigned (6 downto 0) := (others => '0');
	signal graph_uc_data_dbg : std_logic_vector(9 downto 0);

begin


process_pix_load: process (graph_uc_clk, pix_shreg_clr) is
begin
	if (pix_shreg_clr = '1') then
		pix_shreg <= (others => '0');
	elsif (graph_uc_clk'event and graph_uc_clk = '1') then
		pix_shreg(9 downto 0) <= graph_uc_data;
		pix_shreg(19 downto 10) <= pix_shreg(9 downto 0);
		pix_shreg(29 downto 20) <= pix_shreg(19 downto 10);
	end if;
end process process_pix_load;


process_pix_latch: process (clk_in) is
begin
	if (clk_in'event and clk_in = '1') then
		if(pix_shreg(29 downto 27) = "001") then
			pix_loaded <= '1';
		end if;
		if (pix_loaded = '1') then
			pix_addr_buf <= pix_shreg(18 downto 0);
			--if(pix_shreg(0) = '0') then
				pix_color_buf(7 downto 0) <= pix_shreg(26 downto 19);
			--else
				pix_color_buf(15 downto 8) <= pix_shreg(26 downto 19);
			--end if;
			pix_loaded <= '0';
			if (pix_shreg_clr = '0') then
				pix_shreg_clr <= '1';
			end if;
		end if;
		if (pix_shreg_clr = '1') then
			pix_shreg_clr <= '0';
		end if;
		
		
		
		
		if(pix_shreg(29 downto 27) = "010") then
			pix_rect_loaded <= '1';
		end if;
		if (pix_rect_loaded = '1') then
			pix_rect_addr_buf <= pix_shreg(18 downto 0);
			pix_rect_loaded <= '0';
			draw_rect <= '1';
			if (pix_shreg_clr = '0') then
				pix_shreg_clr <= '1';
			end if;
		end if;
	
		
		if(draw_rect = '1') then
			delay_cnt <= delay_cnt + 1;
			if(delay_cnt = 10) then
				delay_cnt <= (others => '0');
				if(pix_addr_buf(9 downto 0) /= pix_rect_addr_buf(9 downto 0)) then
					rect_cnt_x <= rect_cnt_x + 1;
					pix_addr_buf(9 downto 0) <= pix_addr_buf(9 downto 0) + 1;
				else
					pix_addr_buf(9 downto 0) <= pix_addr_buf(9 downto 0) - std_logic_vector(rect_cnt_x);
					rect_cnt_x <= (others => '0');
					if(pix_addr_buf(18 downto 10) /= pix_rect_addr_buf(18 downto 10)) then
						pix_addr_buf(18 downto 10) <= pix_addr_buf(18 downto 10) + 1;
					end if;
				end if;
				if(pix_addr_buf = pix_rect_addr_buf) then
					draw_rect <= '0';
					rect_cnt_x <= (others => '0');
				end if;
			end if;
		end if;
		
	end if;
end process process_pix_latch;




process_mem: process (clk_in) is
begin
	if (clk_in'event and clk_in = '1') then
	
		-- memory write and read
		if (mem_rw_cnt = 0) then
			graph_sram_oe <= '1';
			graph_sram_data_rw <= (others => '0');
			graph_sram_data_wr <= pix_color_buf;
			graph_sram_addr <= pix_addr_buf(18 downto 1);
			if(pix_addr_buf(0) = '0') then
				graph_sram_lb <= '0';
				graph_sram_ub <= '1';
			else
				graph_sram_lb <= '1';
				graph_sram_ub <= '0';
			end if;
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 1) then
			graph_sram_we <= '0';
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 2) then
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 3) then
			graph_sram_we <= '1';
			graph_sram_data_rw <= (others => '1');
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 4) then
			graph_sram_lb <= '0';
			graph_sram_ub <= '0';
			graph_sram_oe <= '0';
			graph_sram_addr <= std_logic_vector(graph_sram_addr_vert) & std_logic_vector(graph_sram_addr_hor);
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 5) then
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 6) then
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 7) then
			tft_data_buf <= graph_sram_data_rd;
			graph_sram_oe <= '1';
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 8) then
			mem_rw_cnt <= mem_rw_cnt + 1;
		elsif (mem_rw_cnt = 9) then
			mem_rw_cnt <= (others => '0');
		end if;
		
		-- tft display clock
		if (mem_rw_cnt = 8) then
			tft_clk_sig <= '1';
		elsif (mem_rw_cnt = 0) then
			tft_clk_sig <= '0';
		elsif (mem_rw_cnt = 3) then
			tft_clk_sig <= '1';
		elsif (mem_rw_cnt = 5) then
			tft_clk_sig <= '0';
		end if;
		
		-- tft display data		
		if (mem_rw_cnt = 8 or mem_rw_cnt = 3) then
			if (tft_vcnt < 510 or (tft_vcnt = 510 and tft_hcnt < 863)) then
			
				if (tft_hcnt < 863) then
					if (tft_hcnt >= 42 and mem_rw_cnt = 8) then
						graph_sram_addr_hor <= tft_hcnt(9 downto 1) - 21;		-- new ram address at each even pixel, starting from 0, after blanking (so the 44/2 subtracted)
					end if;
					if (tft_hcnt >= 44 and mem_rw_cnt = 8) then
						tft_data(5 downto 3) <= tft_data_buf(5 downto 3);
						tft_data(8 downto 6) <= tft_data_buf(7 downto 6) & "0";
						tft_data(2 downto 0) <= tft_data_buf(2 downto 0);
					elsif (tft_hcnt >= 44 and mem_rw_cnt = 3) then
						tft_data(2 downto 0) <= tft_data_buf(10 downto 8);
						tft_data(5 downto 3) <= tft_data_buf(13 downto 11);
						tft_data(8 downto 6) <= tft_data_buf(15 downto 14) & "0";
					end if;
					tft_hcnt <= tft_hcnt + 1;
				else
					tft_hcnt <= (others => '0');
					tft_vcnt <= tft_vcnt + 1;
				end if;
				
				if (tft_vcnt >= 23) then
					graph_sram_addr_vert <= tft_vcnt - 23;
				else
					graph_sram_addr_vert <= (others => '0');
				end if;
				
			else
				tft_vcnt <= (others => '0');
				tft_hcnt <= (others => '0');
			end if;
		end if;
		
	end if;
end process process_mem;


tft_hs <= 	'0' when tft_hcnt = 0 else
				'1';
tft_vs <= 	'0' when tft_vcnt = 0 else
				'1';
tft_de <= '1';
tft_clk <= tft_clk_sig;
graph_sram_cs <= '0';

end Behavioral;
