#include "dsp.h"

extern struct meas_config_struct meas_config;
extern struct cal_struct cal;
extern int g_meas_stop;
extern int g_cal_pt;

//void __attribute__((vector(_CHANGE_NOTICE_VECTOR), interrupt(ipl7soft), nomips16)) dsp_interrupt()
//{
//    IFS1bits.CNCIF = 0; // Clear CN interrupt
//}

void measure(void)
{
    float meas_result[REC_LEN];     //buffer for measurements result
    int i_meas;
    int step_density[] = {3, 30, 300, 3000};    // for step 1, memory length 1k and frequency range 0-3G the frequency step is 1M - this would be good with ideal BRW filter 1MHz (3000 steps with 1MHz width gives the full range) but some overlap (e.g. 3) is needed because filter is not ideal; the step density must be increased when narrower filter is set (in fpga and hardware)
    double freq;
    int64_t meas_raw;   //value received from fpga
    double meas_db;
    int rbw_sel;
    double meas_point_result;
    double f_start, f_stop;
    int i_lo1_lvl_corr, last_i_lo1_lvl_corr, i_gain_corr;
    int meas_points;
    int save_point_index;
    
    g_meas_stop = 0;    //clear the flag which is used to interrupt sweep
    for(i_meas = 0; i_meas < REC_LEN; ++i_meas)
        meas_result[i_meas] = -1000000;    //clear measurements table
    rbw_sel = meas_config.rbw_sel;  //latch filter setting so it's not changed while sweep (would cause bad indexes)
    f_start = meas_config.f_start;  //latch start frequency ...
    f_stop = meas_config.f_stop;  //latch stop frequency ...
    //if(!rbw_sel) LATBbits.LATB3 = 0; else LATBbits.LATB3 = 1;   //set analog filter
    LATFbits.LATF0 = meas_config.if_gain & 0x2;     //set analog gain
    LATFbits.LATF1 = meas_config.if_gain & 0x1;     //set analog gain
    atten_set(meas_config.atten);
    
    // number of points
    // example: if sweep is 0-3GHz with RBW 1MHz then 3000*3 samples are needed so that step is 3x smaller than width of the filter (to reduce amplitude error caused by not ideal flatness of pass-band)
    // if filter is narrower more points is needed (e.g. 10x more for 100kHz to sweep from 0 to 3GHz)
    // if frequency range is lower, less points are needed to have the same step, relative to RBW filter
    meas_points = REC_LEN * step_density[rbw_sel] * (f_stop - f_start) / (FREQ_IN_MAX - FREQ_IN_MIN);
    
    for(i_meas = 0; i_meas < meas_points; ++i_meas)
    {
        // setup pll, measure, wait for result from fpga
        freq = f_start + i_meas * (f_stop - f_start) / (meas_points - 1);
        pll_lo1_set((freq + cal.f_cal + FREQ_IF1) / 2);
        
        i_lo1_lvl_corr = (int)((LVL_CORR_POINTS - 1) * (freq / FREQ_IN_MAX));
        if(i_lo1_lvl_corr != last_i_lo1_lvl_corr)   //if local oscillator frequency changed so that next level calibration point is applicable, send new level to DAC that adjusts LO1 level
        {
            lvl_lo1_set(cal.lo1_lvl_corr[i_lo1_lvl_corr]);
            last_i_lo1_lvl_corr = i_lo1_lvl_corr;
        }
        
        for(int32_t i_delay = 1000; i_delay > 0; --i_delay);
        while(!PORTBbits.RB2);      //check if LO1 pll is locked
        for(int32_t i_delay = 1000; i_delay > 0; --i_delay);
        
        T1CONbits.ON = 0;   //don't interrupt measurement (it would break uC-FPGA communication)
        start_meas(rbw_sel);    //ask fpga to measure
        while(PORTCbits.RC13);  //wait for signal from fpga
        T1CONbits.ON = 1;   //allow interrupts again
        while(!PORTCbits.RC13);  //wait for signal from fpga
        
        meas_raw = get_meas();  //receive result from fpga
        meas_db = 10.0 * log10((double)meas_raw);   // 10*log10 because result received is voltage squared
        meas_point_result = scale_equalize(meas_db, freq);  //apply appropriate corrections, scales, offsets so that it's ready to be displayed
        
        // if filter is narrower, many measurements are saved under the same index because there's not enough memory to save each result - take maximum
        if(meas_points > REC_LEN)
            save_point_index = i_meas * REC_LEN / meas_points;
        else
            save_point_index = i_meas;
        if(meas_result[save_point_index] < meas_point_result)   //take maximum of measurements at frequencies that fits a place in results array
        {
            meas_result[save_point_index] = meas_point_result;
            disp_result(meas_result, save_point_index, meas_points < REC_LEN ? meas_points : REC_LEN);    // display measurement result
        }
        
        if(g_meas_stop) break;
    }
    if(g_cal_pt) gain_corr_point(meas_result, meas_points < REC_LEN ? meas_points : REC_LEN);   //do calibration at the end of the sweep if requested
}

void start_meas(int filt_stages)
{
    int32_t dsp_start_frame;
    
    dsp_start_frame = 0x3 & filt_stages;
    dsp_start_frame |= 1 << 2;
    TRISE = 0x00;
    LATCbits.LATC15 = 1;
   
    LATDbits.LATD11 = 0;
    PORTE = 0;
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    PORTE = 0;
    LATDbits.LATD11 = 1;
    LATCbits.LATC14 = 0;
    LATCbits.LATC14 = 1;
    
    LATDbits.LATD11 = 0;
    PORTE = 0xFF & (dsp_start_frame >> 8);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    PORTE = 0xFF & (dsp_start_frame >> 0);
    LATDbits.LATD11 = 1;
    LATCbits.LATC14 = 0;
    LATCbits.LATC14 = 1;
}

int64_t get_meas(void)
{
    int64_t dsp_result_frame = 0;
    
    TRISE = 0xFF;
    LATCbits.LATC15 = 0;
    LATDbits.LATD11 = 0;
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 5);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 4);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 3);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 2);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 1);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    dsp_result_frame |= (int64_t)(PORTE & 0xFF) << (8 * 0);
    LATDbits.LATD11 = 1;
    LATDbits.LATD11 = 0;
    LATCbits.LATC15 = 1;
    TRISE = 0x00;
    
    return dsp_result_frame;
}

float scale_equalize(double meas_db, double freq)
{
    int i_gain_corr;
    double fir_scale;
    double gain_corr;
    
    //this correction of fir filter gain may be incorrect - TODO - check fpga code
    fir_scale = 3.010 + meas_config.rbw_sel * 3.803;    //3.803 should be from 10x downsampling and some 2^n multiplication when assigning vectors, additional 3dB would be added because of mixing signal with sin and cos although I've no idea what I'm doing
    
    i_gain_corr = llround(GAIN_CORR_POINTS * freq / (FREQ_IN_MAX - FREQ_IN_MIN) - 1);    //choose amplitude correction point for a given frequency
    if(i_gain_corr < 0) i_gain_corr = 0;    //clamp index at 0
    
    //interpolate correction of input attenuator attenuation nonlinearity (it's calibrated at 0, 10, 20, 30dB at each frequency point, others are calculated here)
    if(meas_config.atten <= 10)
        gain_corr = ((10 - meas_config.atten) * cal.gain_corr[i_gain_corr][0] + meas_config.atten * cal.gain_corr[i_gain_corr][1]) / 10;
    else if(meas_config.atten <= 20)
        gain_corr = ((10 - (meas_config.atten - 10)) * cal.gain_corr[i_gain_corr][1] + (meas_config.atten - 10) * cal.gain_corr[i_gain_corr][2]) / 10;
    else if(meas_config.atten <= 30)
        gain_corr = ((10 - (meas_config.atten - 20)) * cal.gain_corr[i_gain_corr][2] + (meas_config.atten - 20) * cal.gain_corr[i_gain_corr][3]) / 10;
    
    return meas_db - MEAS_OFFS - meas_config.ref_lvl + gain_corr - fir_scale;   //meas offset is just to place the plot roughly where it should be on the screen
}

void gain_corr_point(float *meas_result, int n_points)
{
    int i_gain_corr;
    int i_pk_search;
    float peak = -1000000;
    static float correct_atten_buf[4];
    static int i_atten = 0;
    int i_cpy;
    
    i_gain_corr = llround(GAIN_CORR_POINTS * ((meas_config.f_stop + meas_config.f_start) / 2) / (FREQ_IN_MAX - FREQ_IN_MIN) - 1);    //calculate index in calibration coefficients array based on frequency
    for(i_pk_search = 0; i_pk_search < n_points; ++i_pk_search)     //find peak - the signal used to calibrate level at the frequency
        if(meas_result[i_pk_search] > peak)
            peak = meas_result[i_pk_search];

    if(i_atten > 0) //skip i_atten == 0 so that calibration is done after one full sweep
        correct_atten_buf[i_atten - 1] = (CAL_LVL_DBM - meas_config.atten) - peak;  //at each attenuation level (0, 10, 20, 30) the difference between measured level and level of calibration signal is calculated
    meas_config.atten = 10 * i_atten;   //next attenuation to be calibrated
    ++i_atten;
    if(i_atten == 5)    //at the end save calibration factor for the given frequency
    {
        g_cal_pt = 0;
        i_atten = 0;
        for(i_cpy = 0; i_cpy < 4; ++i_cpy) cal.gain_corr[i_gain_corr][i_cpy] = correct_atten_buf[i_cpy];
        meas_config.atten = 0;
    }
}

void gain_corr_clr(void)
{
    int i_gain_corr;
    for(i_gain_corr = 0; i_gain_corr < GAIN_CORR_POINTS; ++i_gain_corr)
    {
        cal.gain_corr[i_gain_corr][0] = 0;  //0dB
        cal.gain_corr[i_gain_corr][1] = 0;  //10dB
        cal.gain_corr[i_gain_corr][2] = 0;  //20dB
        cal.gain_corr[i_gain_corr][3] = 0;  //30dB
    }
}

void gain_corr_apply(void)
{
    int i_interp;
    int i_indexes;
    int ind_arr_last;
    int i_start, i_stop;
    int cal_indexes[GAIN_CORR_POINTS];    // only some of the points are calibrated, the rest is interpolated
    float a[4];
    
    i_indexes = -1;
    for(i_interp = 0; i_interp < GAIN_CORR_POINTS; ++i_interp)  //find and list all the points which were calibrated (not zero)
        if(cal.gain_corr[i_interp][0] != 0)
            cal_indexes[++i_indexes] = i_interp;
    ind_arr_last = --i_indexes;

    i_start = 0;
    i_stop = 1;
    for(i_interp = 0; i_interp < GAIN_CORR_POINTS; ++i_interp)
    {
        if(i_interp < cal_indexes[0]) //all points below the first are made the same as the first
        {
            cal.gain_corr[i_interp][0] = cal.gain_corr[cal_indexes[0]][0];
            cal.gain_corr[i_interp][1] = cal.gain_corr[cal_indexes[0]][1];
            cal.gain_corr[i_interp][2] = cal.gain_corr[cal_indexes[0]][2];
            cal.gain_corr[i_interp][3] = cal.gain_corr[cal_indexes[0]][3];
        }
        else if(i_interp > cal_indexes[ind_arr_last]) //all points above the last are made the same as the last
        {
            cal.gain_corr[i_interp][0] = cal.gain_corr[cal_indexes[ind_arr_last]][0];
            cal.gain_corr[i_interp][1] = cal.gain_corr[cal_indexes[ind_arr_last]][1];
            cal.gain_corr[i_interp][2] = cal.gain_corr[cal_indexes[ind_arr_last]][2];
            cal.gain_corr[i_interp][3] = cal.gain_corr[cal_indexes[ind_arr_last]][3];
        }
        else    //interpolation
        {
            if(i_interp > cal_indexes[i_start] && i_interp < cal_indexes[i_stop])
            {
                a[0] = (cal.gain_corr[cal_indexes[i_stop]][0] - cal.gain_corr[cal_indexes[i_start]][0]) / (cal_indexes[i_stop] - cal_indexes[i_start]);
                a[1] = (cal.gain_corr[cal_indexes[i_stop]][1] - cal.gain_corr[cal_indexes[i_start]][1]) / (cal_indexes[i_stop] - cal_indexes[i_start]);
                a[2] = (cal.gain_corr[cal_indexes[i_stop]][2] - cal.gain_corr[cal_indexes[i_start]][2]) / (cal_indexes[i_stop] - cal_indexes[i_start]);
                a[3] = (cal.gain_corr[cal_indexes[i_stop]][3] - cal.gain_corr[cal_indexes[i_start]][3]) / (cal_indexes[i_stop] - cal_indexes[i_start]);
                cal.gain_corr[i_interp][0] = cal.gain_corr[cal_indexes[i_start]][0] + (i_interp - cal_indexes[i_start]) * a[0];            
                cal.gain_corr[i_interp][1] = cal.gain_corr[cal_indexes[i_start]][1] + (i_interp - cal_indexes[i_start]) * a[1];   
                cal.gain_corr[i_interp][2] = cal.gain_corr[cal_indexes[i_start]][2] + (i_interp - cal_indexes[i_start]) * a[2];   
                cal.gain_corr[i_interp][3] = cal.gain_corr[cal_indexes[i_start]][3] + (i_interp - cal_indexes[i_start]) * a[3];                   
            }
            if(i_interp == cal_indexes[i_stop])
            {
                i_start = i_stop;
                ++i_stop;
            }
        }
    }
}
