#include "flash.h"

extern struct cal_struct cal;

unsigned int NVMUnlock (unsigned int nvmop) //based on https://ww1.microchip.com/downloads/en/DeviceDoc/60001121g.pdf
{
    unsigned int status;
    
    asm volatile ("di %0" : "=r" (status));// Suspend or Disable all Interrupts
    
    NVMCON = nvmop;     // Enable Flash Write/Erase Operations and Select Flash operation to perform
    NVMKEY = 0xAA996655;    // Write Keys
    NVMKEY = 0x556699AA;    // Write Keys
    NVMCONSET = 0x8000;    // Start the operation using the Set Register
    while (NVMCON & 0x8000);    // Wait for operation to complete
    
    if(status & 0x00000001)    // Restore Interrupts1
        {asm volatile ("ei");}
    else
        {asm volatile ("di");}
    NVMCONCLR = 0x0004000;    // Disable NVM write enable

    return (NVMCON & 0x3000);    // Return WRERR and LVDERR Error Status Bits
}

unsigned int NVMWriteWord (int32_t address, int32_t data) //based on https://ww1.microchip.com/downloads/en/DeviceDoc/60001121g.pdf
{
    unsigned int res;

    NVMDATA = data;    // Load data into NVMDATA register
    NVMADDR = address;    // Load address to program into NVMADDR register
    res = NVMUnlock (0x4001);    // Unlock and Write Word
    return res;    // Return Result
}

unsigned int NVMErasePage(int32_t address) //based on https://ww1.microchip.com/downloads/en/DeviceDoc/60001121g.pdf
{
    unsigned int res;
    NVMADDR = address;    // Set NVMADDR to the Start Address of page to erase
    res = NVMUnlock(0x4004);    // Unlock and Erase Page
    return res;    // Return Result
}

void save_cal(void) //save structure with calibration data to nonvolatile memory
{
    int nv_data_addr_phy;
    void *conf_ptr;
    int32_t wr_word;
    int i_save;
    
    nv_data_addr_phy = 0x1D00F000;  //use the last page (4kB) - physical address to write using NVMWriteWord
    conf_ptr = (void*)&cal; //address at which calibration structure starts
    
    NVMErasePage(nv_data_addr_phy);
    for(i_save = 0; i_save < sizeof(cal) / 4; ++i_save)     // "sizeof(cal) / 4" is a size in 32bit words, not bytes
    {
        wr_word = *((int32_t*)conf_ptr + i_save);       //save the struct in chunks 32bit wide
        NVMWriteWord(nv_data_addr_phy + i_save * 4, wr_word);   //index to save words is multiplied by 4 because 32bit words are saved (4x longer than byte)
    }
}

void load_cal(void) //load structure with calibration data from nonvolatile memory
{
    int *nv_data_addr_virt_ptr;
    void *conf_ptr;
    int i_read;
    int32_t rd_word;
    
    nv_data_addr_virt_ptr = (int*)0x9D00F000;   //virtual address at which calibration structure starts in nonvolatile memory
    conf_ptr = (void*)&cal;   //address at which calibration structure (in RAM) starts

    for(i_read = 0; i_read < sizeof(cal) / 4; ++i_read)
    {
        rd_word = *(nv_data_addr_virt_ptr + i_read);
        *((int32_t*)conf_ptr + i_read) = rd_word;
    }
}
