#include "graph.h"

struct pix_struct pix;



void pixel(struct pix_struct pix)
{
    int32_t pix_frame;
    pix_frame = 0;
    pix_frame |= pix.x << 0;
    pix_frame |= pix.y << 10;
    pix_frame |= pix.r << 19;
    pix_frame |= pix.g << 22;
    pix_frame |= pix.b << 25;
    pix_frame |= 0x1   << 27;
    
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 20;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 10;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 0;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
}
/* //slow
void draw_rect(struct pix_struct pix_start, struct pix_struct pix_stop)
{
    int i_draw_x;
    int i_draw_y;
    pix.r = pix_start.r;
    pix.g = pix_start.g;
    pix.b = pix_start.b;
    for(i_draw_x = pix_start.x; i_draw_x <= pix_stop.x; ++i_draw_x)
    {
        pix.x = i_draw_x;
        for(i_draw_y = pix_start.y; i_draw_y <= pix_stop.y; ++i_draw_y)
        {
            pix.y = i_draw_y;
            pixel(pix);
        }
    }
}*/

void draw_rect(struct pix_struct pix_start, struct pix_struct pix_stop)
{
    int32_t pix_frame;
    pix_frame = 0;
    
    pix.r = pix_start.r;
    pix.g = pix_start.g;
    pix.b = pix_start.b;
    pix.x = pix_start.x;
    pix.y = pix_start.y;
    pixel(pix);     // first pixel sent as usual
                    // second pixel (2nd corner of rectangle) is sent the same way except with 28 bit set instead of 27
    pix.x = pix_stop.x;
    pix.y = pix_stop.y;
    
    pix_frame |= pix.x << 0;
    pix_frame |= pix.y << 10;
    pix_frame |= pix.r << 19;
    pix_frame |= pix.g << 22;
    pix_frame |= pix.b << 25;
    pix_frame |= 0x1   << 28;
    
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 20;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 10;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
    LATD = 0x3FF & pix_frame >> 0;
    LATDbits.LATD10 = 1;
    LATDbits.LATD10 = 0;
    
    for(int32_t i_delay = (pix_stop.x - pix_start.x + 1) * (pix_stop.y - pix_start.y + 1) / 2; i_delay > 0; --i_delay);
}

void draw_rect_slow(struct pix_struct pix_start, struct pix_struct pix_stop)
{
    int i_draw_x;
    int i_draw_y;
    int32_t pix_frame;
    int32_t pix_frame_tmp;
    int32_t pix_frame_tmp1, pix_frame_tmp2;
    
    pix_frame_tmp = 0;
    pix_frame_tmp |= pix_start.r << 19;
    pix_frame_tmp |= pix_start.g << 22;
    pix_frame_tmp |= pix_start.b << 25;
    pix_frame_tmp |= 0x1   << 27;
    
    for(i_draw_y = pix_start.y; i_draw_y <= pix_stop.y; ++i_draw_y)
    {
        pix_frame = pix_frame_tmp | (i_draw_y << 10) /*| i_draw_x*/;
        pix_frame_tmp1 = 0x3FF & pix_frame >> 20;
        pix_frame_tmp2 = 0x3FF & pix_frame >> 10;
        
        for(i_draw_x = pix_start.x; i_draw_x <= pix_stop.x; ++i_draw_x)
        {
            pix_frame = /*pix_frame_tmp |*/ (i_draw_y << 10) | i_draw_x;
            LATD = pix_frame_tmp1;
            LATDbits.LATD10 = 1;
            LATD = pix_frame_tmp2;
            LATDbits.LATD10 = 1;
            LATD = 0x3FF & pix_frame;
            LATDbits.LATD10 = 1;
        }
    }
}

void draw_char(char character, struct pix_struct pix_start)
{
    int i_draw_x;
    int i_draw_y;
    for(i_draw_x = 0; i_draw_x < CHAR_W; ++i_draw_x)
    {
        for(i_draw_y = 0; i_draw_y < CHAR_H; ++i_draw_y)
        {
            if((ascii_lut[(int)character][i_draw_x] >> i_draw_y) & 0x1 == 1)
            {
                pix.r = pix_start.r;
                pix.g = pix_start.g;
                pix.b = pix_start.b;
            }
            else    //TODO - text on black background only - fix?
            {
                pix.r = 0;
                pix.g = 0;
                pix.b = 0;
            }
            pix.x = pix_start.x + i_draw_x;
            pix.y = pix_start.y + i_draw_y;
            pixel(pix);
        }
    }
}

void draw_text(char *str_to_draw, struct pix_struct pix_start)
{
    int i_strdrw;
    for(i_strdrw = 0; str_to_draw[i_strdrw] != '\0'; ++i_strdrw)
    {
        pix_start.x += CHAR_W + 1;
        draw_char(str_to_draw[i_strdrw], pix_start);
    }
}

void print_float(float fnum, int dp, struct pix_struct pix_printfloat)
{
    int64_t int_val;
    int i_div = 0;
    int64_t pow10div;
    int tmp_digit;
    int nstarted = 0;
    char num_text[12];
    
    int_val = llround(fabs(fnum) * pow(10, dp));
    if(int_val == 0)
    {
        num_text[0] = ' ', num_text[1] = '0', num_text[2] = '\0';
        draw_text(num_text, pix_printfloat);
        return;
    }
    if(fnum > 0) num_text[0] = ' ';
    else num_text[0] = '-';
    
    for(pow10div = 1000000000000; pow10div >= 1; pow10div /= 10)
    {
        tmp_digit = int_val / pow10div;
        int_val = int_val - tmp_digit * pow10div;
        if(tmp_digit > 0) nstarted = 1;
        if(nstarted)
        {
            num_text[++i_div] = tmp_digit + '0';
            if(pow10div == pow(10, dp) && dp) num_text[++i_div] = '.';
        }
    }
    num_text[++i_div] = '\0';
    draw_text(num_text, pix_printfloat);
}

void disp_result(float *meas_result, int meas_point, int n_points)
{
    int pix_x, prev_pix_x;
    static int last_pix_x = -1;    
    static struct pix_struct pix_res = {0, 0, 3, 3, 3};   //init with white color
    static struct pix_struct pix_clr_start = {0, 0, 0, 0, 0};   //init with black color
    static struct pix_struct pix_clr_stop;
    static struct pix_struct pix_grid_start = {0, 0, 0, 1, 0};   //init with green color
    static struct pix_struct pix_grid_stop;
    int dx, dy, i_dx, i_dy;
    int i_grid_x, i_grid_y;
    double tg;
    T1CONbits.ON = 0;       // it's necessary to prevent drawing on screen simultaneously by the following code and ISR (user interface)
	if(n_points > PLOT_W)       // drawing measurement is done slightly differently if more points is squeezed to screen width and differently if there's less points and interpolation is needed
	{
		pix_x = PLOT_W * meas_point / n_points;
		prev_pix_x = PLOT_W * (meas_point - 1) / n_points;
		dy = (int32_t)(PIX_PER_DB * meas_result[meas_point]) - (int32_t)(PIX_PER_DB * meas_result[meas_point - 1]);
        
        // draw grid
        if(pix_x != last_pix_x)
        {
            if(!(pix_x % ((PLOT_W - 1) / 10)))
            {
                pix_grid_start.x = PLOT_START_X + pix_x;
                pix_grid_stop.x = pix_grid_start.x;
                pix_grid_start.y = PLOT_START_Y;
                pix_grid_stop.y = PLOT_STOP_Y;
                draw_rect(pix_grid_start, pix_grid_stop);
            }
            else
            {
                pix_grid_start.x = PLOT_START_X + pix_x;
                for(i_grid_y = PLOT_START_Y; i_grid_y <= PLOT_STOP_Y; i_grid_y += PLOT_H / 10)
                {
                    pix_grid_start.y = i_grid_y;
                    pixel(pix_grid_start);
                }
            }
            last_pix_x = pix_x;
        }
        
		for(i_dy = 0; i_dy != dy; dy > 0 ? ++i_dy : --i_dy)     //drawing almost vertical line connecting two adjacent points on screen
		{
			if(abs(i_dy) < abs(dy) / 2)
				pix_res.x = PLOT_START_X + pix_x;
			else
				pix_res.x = PLOT_START_X + prev_pix_x;
			pix_res.y = PLOT_STOP_Y + (int32_t)(PIX_PER_DB * meas_result[meas_point]) - i_dy;
			pixel(pix_res);
		}
        pix_clr_start.x = PLOT_START_X + pix_x + 1;
        pix_clr_stop.x = PLOT_START_X + pix_x + 2;
        pix_clr_start.y = PLOT_START_Y;
        pix_clr_stop.y = PLOT_STOP_Y;
        draw_rect(pix_clr_start, pix_clr_stop);    //clear columns for next points
	}
	else
	{
		pix_x = (PLOT_W - 1) * meas_point / (n_points - 1);
		prev_pix_x = (PLOT_W - 1) * (meas_point - 1) / (n_points - 1);
        dx = pix_x - prev_pix_x;
		dy = (int32_t)(PIX_PER_DB * meas_result[meas_point]) - (int32_t)(PIX_PER_DB * meas_result[meas_point - 1]);
		tg = (double)dy / (pix_x - prev_pix_x);
        
        // draw grid
        for(i_grid_x = 0; i_grid_x <= dx; ++i_grid_x)
        {
            pix_grid_start.y = PLOT_START_Y;
            pix_grid_stop.y = PLOT_STOP_Y;
            pix_grid_start.x = PLOT_START_X + pix_x - i_grid_x;
            pix_grid_stop.x = pix_grid_start.x;
            if(!((pix_x - i_grid_x) % ((PLOT_W - 1) / 10)))
                draw_rect(pix_grid_start, pix_grid_stop);
            else if(pix_x > 0)
                for(i_grid_y = PLOT_START_Y; i_grid_y <= PLOT_STOP_Y; i_grid_y += PLOT_H / 10)
                {
                    pix_grid_start.y = i_grid_y;
                    pixel(pix_grid_start);
                }
        }
        
        if(abs(tg) > 0.5 && pix_x > 0)        //drawing line between pixels that can be not adjacent (iterate in y axis if line more vertical or in x if more horizontal)
        {
            for(i_dy = 0; i_dy != dy; dy > 0 ? ++i_dy : --i_dy)
            {
                pix_res.x = PLOT_START_X + pix_x - abs((int)(i_dy / tg));
                pix_res.y = PLOT_STOP_Y + (int32_t)(PIX_PER_DB * meas_result[meas_point]) - i_dy;
                pixel(pix_res);
            }
        }
        else if(pix_x > 0)
        {
            for(i_dx = 0; i_dx != dx; dx > 0 ? ++i_dx : --i_dx)
            {
                pix_res.x = PLOT_START_X + pix_x - i_dx;
                pix_res.y = PLOT_STOP_Y + (int32_t)(PIX_PER_DB * meas_result[meas_point]) - (i_dx * tg);
                pixel(pix_res);
            }
        }
        pix_clr_start.x = PLOT_START_X + pix_x + 1;
        pix_clr_stop.x = PLOT_START_X + pix_x + PLOT_W / n_points + 1;
        pix_clr_start.y = PLOT_START_Y;
        pix_clr_stop.y = PLOT_STOP_Y;
        draw_rect(pix_clr_start, pix_clr_stop);    //clear columns for next points
	}
    T1CONbits.ON = 1;
}
