#include "init.h"

extern struct meas_config_struct meas_config;
extern struct cal_struct cal;

void init_uc(void)
{
    init_clk();
    init_int();
    init_spi();
    init_i2c();
    init_adc();
    init_pins();
    init_timer_keyb();
}

void init_sys(void)
{
    //power on voltage sequence for display
    for(int32_t i_delay = 100000; --i_delay; i_delay > 0);
    LATFbits.LATF3 = 1;
    for(int32_t i_delay = 100000; --i_delay; i_delay > 0);
    LATFbits.LATF2 = 1;
    init_pll_lo2();
    lvl_lo1_set(0x00);
    init_pll_lo1(2000000000);
    
    load_cal();     //take calibration data from nonvolatile memory
    if(cal.n_first_cal != 1)    //if calibration hasn't been done after flashing the uC, initialize calibration data
    {
        cal.f_cal = 0;
        gain_corr_clr();
    //    lvl_lo1_cal();
        cal.n_first_cal = 1;
        save_cal();
    }
    meas_config.atten = 30;
    meas_config.f_start = 0;
    meas_config.f_stop = 3000000000;
    meas_config.ref_lvl = 0;
    meas_config.scale = 10;
    meas_config.rbw_sel = 0;
    meas_config.if_gain = 0;
    atten_set(meas_config.atten);
    T1CONbits.ON = 1;   // enable timer 1
}

void init_clk(void)
{
    //no additional configuration (all done by pragma)
}

void init_int(void)
{
    asm volatile("ei"); // Globally enable interrupts (in Assembly).
    INTCONbits.MVEC = 1;
}

void init_spi(void)
{
    SPI2BRG = 0x1;  // use FPB/4 clock frequency
    SPI2CONbits.MSTEN = 1;  // SPI master
    SPI2CONbits.CKE = 1; // Data change at falling edge
    SPI2CONbits.ON = 1; // enable SPI2
}

void init_i2c(void)
{
    I2C2BRG = 0x0011;   // I2C about 0.4MHz at PBCLK=120M/8
    I2C2CONbits.ON = 1;
}

void init_adc(void)
{
    AD1CON1bits.FORM = 0;   // unsigned integer
    AD1CON1bits.SSRC = 0;   // Clearing SAMP bit ends sampling and starts conversion
    AD1CON1bits.ASAM = 1;   // Sampling begins immediately after last conversion completes; SAMP bit is automatically set.
    AD1CON2bits.VCFG = 0;   // reference is VDD-VSS
    AD1CON2bits.BUFM = 0;   // Buffer configured as one 16-word buffer
    AD1CON3bits.ADRC = 0;   // Clock derived from Peripheral Bus Clock (PBCLK)
    AD1CHSbits.CH0NA = 0;   // Channel 0 negative input is V REFL
    AD1CHSbits.CH0SA = 0x04;// ADC at AN4 (level in from 1st LO)
    AD1CON1bits.ON = 1;     // enable adc
}

void init_timer_keyb(void)
{
    T1CONbits.TCKPS = 0x3;  // prescaler 256
    T1CONbits.TCS = 0x0;    // internal clock
    PR1 = 0xFF;     // frequency about 120000000/8/256/256 = 229Hz
    IPC1bits.T1IP = 1;  //priority 1
    IFS0bits.T1IF = 0; // Reset timer1 interrupt
    IEC0bits.T1IE = 1; // Enable CN interrupts
//    T1CONbits.ON = 1;   // enable timer 1
}
    
void init_pins(void)
{
    ANSELB = 0;
    ANSELC = 0;
    ANSELD = 0;
    ANSELE = 0;
    ANSELF = 0;
    ANSELG = 0;
    TRISBbits.TRISB2 = 1;   // output - extra pin to IF control - changed in hardware to input from IF1 pll lock
    TRISBbits.TRISB3 = 0;   // output to control narrower IF filter
    TRISBbits.TRISB4 = 1;   // level analog input
    TRISBbits.TRISB5 = 0;   // output to set 1st range of 1st LO frequencies
    TRISBbits.TRISB6 = 0;   // output to set 2nd range of 1st LO frequencies
    TRISBbits.TRISB7 = 0;   // output to set 3rd range of 1st LO frequencies
    TRISBbits.TRISB8 = 0;   // keyboard - latch output
    TRISBbits.TRISB9 = 0;   // keyboard - clock output
    TRISBbits.TRISB10 = 1;   // keyboard - rotary switch negative input
    TRISBbits.TRISB11 = 1;   // keyboard - rotary switch positive input
    TRISBbits.TRISB12 = 1;   // keyboard - data input
    TRISBbits.TRISB13 = 0;   // keyboard - select group bit 0 output
    TRISBbits.TRISB14 = 0;   // keyboard - select group bit 1 output   
    TRISBbits.TRISB15 = 1;   // 12V power sense analog input
    
    TRISCbits.TRISC13 = 1;   // input - fpga communication (interrupt from fpga)
    TRISCbits.TRISC14 = 0;   // output - fpga communication (wr)
    TRISCbits.TRISC15 = 0;   // output - fpga communication (dir)
    
    TRISD &= ~0x3FF;   // output - graphics data to fpga
    TRISDbits.TRISD10 = 0;   // output - graphics clk to fpga
    TRISDbits.TRISD11 = 0;   // output - fpga communication clk
    
    TRISE = 0x00;   // output - data to fpga

    TRISFbits.TRISF0 = 0;   // output to control 2nd IF controllable amplifier
    TRISFbits.TRISF1 = 0;   // output to control 1st IF controllable amplifier
    TRISFbits.TRISF2 = 0;   // output to control positive TFT voltage
    TRISFbits.TRISF3 = 0;   // output to control negative TFT voltage
    LATFbits.LATF2 = 0;
    LATFbits.LATF3 = 0;
    
    TRISGbits.TRISG3 = 0;   // output to control attenuator F1951 LE pin
    TRISGbits.TRISG8 = 0;   // output to control PLL2 LE pin
    TRISGbits.TRISG9 = 0;   // output to control PLL1 LE pin

    ANSELBbits.ANSB4 = 1;   // level analog input
    ANSELBbits.ANSB15 = 1;   // 12V power sense analog input
    
    // unused - pull-up
    CNPUBbits.CNPUB0 = 1;
    CNPUBbits.CNPUB1 = 1;
    CNPUBbits.CNPUB10 = 1;
    CNPUBbits.CNPUB11 = 1;
    CNPUBbits.CNPUB12 = 1;
    CNPUBbits.CNPUB13 = 1;
    CNPUBbits.CNPUB14 = 1;
    CNPUFbits.CNPUF0 = 1;
    CNPUFbits.CNPUF1 = 1;
    CNPUFbits.CNPUF6 = 1;
    CNPUGbits.CNPUG2 = 1;
    CNPUGbits.CNPUG3 = 1;
    
    RPG7R = 0x06;   // map SPI 2 SDO to pin RPG7
}
