#include "pll.h"

void init_pll_lo2(void)     //3772MHz LO PLL config
{
    int32_t r0, r1, r2, r3, r2i; //register values for pll
    r0 = 0x00000000;
    r0 =    0x0UL << 23 | //no cycle slip reduction
            0x96UL << 14 | //integer divider of RF input
            0x58UL << 2 | //fractional divider of RF input - numerator
            0x0UL; //address of r0
    r1 = 0x00000000;
    r1 =    0x0UL << 23 | //load modulus
            0x1UL << 20 | //lock detect to mux output
            0x1UL << 18 | //prescaller 8/9
            0x1UL << 14 | //no ref division
            0x64UL << 2 | //fractional divider of RF input - denominator
            0x1UL; //address of r1
    r2i = 0x00000000;
    r2 = 0x00000000;
    r2i =   0x1UL << 12 | //no resync (phase synchronization)
            0x0UL << 11 | //no ref doubler
            0x3UL << 7 | //charge pump setting - 2.5???
            0x1UL << 6 | //positive vco characteristics
            0x0UL << 5 | //24 cycles to lock detect
            0x0UL << 4 | //no power down
            0x0UL << 3 | //no 3state charge pump
            0x1UL << 2 | //reset
            0x2UL; //address of r2
    r2 =    0x1UL << 12 | //no resync (phase synchronization)
            0x0UL << 11 | //no ref doubler
            0x3UL << 7 | //charge pump setting - 2.5???
            0x1UL << 6 | //positive vco characteristics
            0x0UL << 5 | //24 cycles to lock detect
            0x0UL << 4 | //no power down
            0x0UL << 3 | //no 3state charge pump
            0x0UL << 2 | //no reset
            0x2UL; //address of r2
    r3 = 0x00000000;
    r3 =    0x0UL << 2 | //init r3
            0x3UL; //address of r2

    LATGbits.LATG8 = 1;
    LATGbits.LATG8 = 0;
    SPI2BUF = 0xff & (r3 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG8 = 1;
    LATGbits.LATG8 = 0;
//    SPI2BUF = 0xff & (r2i >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2i >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2i >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG8 = 1;
    LATGbits.LATG8 = 0;
    SPI2BUF = 0xff & (r1 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG8 = 1;
    LATGbits.LATG8 = 0;
    SPI2BUF = 0xff & (r0 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG8 = 1;
    LATGbits.LATG8 = 0;
    SPI2BUF = 0xff & (r2 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG8 = 1;
}

void init_pll_lo1(double freq)     // 2000-3500MHz LO PLL config
{
    int32_t r0, r1, r2, r3, r4; //register values for pll
    int32_t integ; //integer divider of RF input
    int32_t frac; //fractional divider of RF input

    integ = (int32_t) (freq / FREF);
    frac = (int32_t) (((freq / FREF) - integ) * 0x2000000);

    if (freq < FVCO1_STOP) {
        LATBbits.LATB5 = 1;
        LATBbits.LATB6 = 0;
        LATBbits.LATB7 = 0;
    } else if (freq < FVCO2_STOP) {
        LATBbits.LATB5 = 0;
        LATBbits.LATB6 = 1;
        LATBbits.LATB7 = 0;
    } else {
        LATBbits.LATB5 = 0;
        LATBbits.LATB6 = 0;
        LATBbits.LATB7 = 1;
    }

    r0 = 0x00000000;
    r0 =    0x5UL << 27 | //analog lock detect to mux output
            integ << 15 | //integer divider of RF input
            (0xfff & (frac >> 13)) << 3 | //msb part of fractional divider of RF input
            0x0; //address of r0
    r1 = 0x00000000;
    r1 =    (0x1fff & frac) << 15 | //lsb part of fractional divider of RF input
            0x1; //address of r1
    r2 = 0x00000000;
    r2 =    0x0UL << 28 | //no cycle slip reduction
            0x7UL << 24 | //charge pump setting - 2.5???
            0x1UL << 22 | //prescaller 8/9
            0x1UL << 21 | // R divider enabled!!!!!!
            0x0UL << 20 | //no ref doubler
            0x1UL << 15 | //no input reference division
            0x2; //address of r2
    r3 = 0x00000000;
    r3 =    0x0UL << 14 | //sig-del reset enable
            0x0UL << 7 | //24 cycles to lock detect
            0x1UL << 6 | //positive vco characteristics
            0x0UL << 5 | //no power down
            0x0UL << 4 | //no 3state charge pump
            0x0UL << 3 | //no reset
            0x3; //address of r3
    r4 = 0x00000000;
    r4 =    0x3UL << 23 | //negative bleed current on
            0x0UL << 19 | //no clock divider mode (fastlock??)
            0x000UL << 7 | //something with fastlock, don't care ??
            0x4; //address of r4

    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r4 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r3 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r2 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r1 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r0 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
}

void pll_lo1_set(double freq)
{
    int32_t r0, r1, r2, r3, r4; //register values for pll
    int32_t integ; //integer divider of RF input
    int32_t frac; //fractional divider of RF input

    integ = (int32_t) (freq / FREF);
    frac = (int32_t) (((freq / FREF) - integ) * 0x2000000);

    if (freq < FVCO1_STOP) {
        LATBbits.LATB5 = 1;
        LATBbits.LATB6 = 0;
        LATBbits.LATB7 = 0;
    } else if (freq < FVCO2_STOP) {
        LATBbits.LATB5 = 0;
        LATBbits.LATB6 = 1;
        LATBbits.LATB7 = 0;
    } else {
        LATBbits.LATB5 = 0;
        LATBbits.LATB6 = 0;
        LATBbits.LATB7 = 1;
    }

    r0 = 0x00000000;
    r0 =    0x6UL << 27 | //digital lock detect to mux output
            integ << 15 | //integer divider of RF input
            (0xfff & (frac >> 13)) << 3 | //msb part of fractional divider of RF input
            0x0; //address of r0
    r1 = 0x00000000;
    r1 =    (0x1fff & frac) << 15 | //lsb part of fractional divider of RF input
            0x1; //address of r1
    r2 = 0x00000000;
    r2 =    0x0UL << 28 | //no cycle slip reduction
            0x7UL << 24 | //charge pump setting - 2.5???
            0x1UL << 22 | //prescaller 8/9
            0x1UL << 21 | // R divider enabled!!!!!!
            0x0UL << 20 | //no ref doubler
            0x1UL << 15 | //no input reference division
            0x2; //address of r2
    r3 = 0x00000000;
    r3 =    0x0UL << 14 | //sig-del reset enable
            0x0UL << 7 | //24 cycles to lock detect
            0x1UL << 6 | //positive vco characteristics
            0x0UL << 5 | //no power down
            0x0UL << 4 | //no 3state charge pump
            0x0UL << 3 | //no reset
            0x3; //address of r3
    r4 = 0x00000000;
    r4 =    0x0UL << 23 | //negative bleed current off
            0x0UL << 19 | //no clock divider mode (fastlock??)
            0x000UL << 7 | //something with fastlock, don't care ??
            0x4; //address of r4

    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r4 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r4 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r3 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r3 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r2 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r2 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r1 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r1 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
    LATGbits.LATG9 = 0;
    SPI2BUF = 0xff & (r0 >> 24);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 16);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 8);
    while(SPI2STATbits.SPIBUSY);
    SPI2BUF = 0xff & (r0 >> 0);
    while(SPI2STATbits.SPIBUSY);
    LATGbits.LATG9 = 1;
}
