#include "ui.h"
#include "lvl_lo1.h"

extern struct meas_config_struct meas_config;
extern struct cal_struct cal;
extern int g_meas_stop;
extern int g_cal_pt;


void __attribute__((vector(_TIMER_1_VECTOR), interrupt(ipl1soft), nomips16)) keyb_interrupt()
{
    static int pressed_key = K_CONTEXT5;   //initialize with exit button to redraw main menu
    static int last_key;   //it's to do action only when key is pressed (change state)
    static int menu = -1;   //
    static int last_menu;      //remember where to return after exiting submenu
    static char numb_enter_str[STR_BUF_LEN];
    static int i_numb_enter;
    static int i_numb_conv;
    static int lock_dot;
    static double enter_num;
    static int dot_pos;
    static int number_ready = 0;
    static struct pix_struct pix_enter_num = {NUM_ENTER_FIELD_X, 440, 0x3, 0x0, 0x0};
    static int enter_num_cursor = NUM_ENTER_FIELD_X + (CHAR_W + 1);
    static int sel_num_enter;
    struct pix_struct pix_start, pix_stop;
    
    if(menu == -1)  // initial screen content
    {
        pix_stop.x = SCREEN_W + (pix_start.x = 0);
        pix_stop.y = SCREEN_H + (pix_start.y = 0);
        pix_start.r = pix_start.g = pix_start.b = 0x0;
        draw_rect(pix_start, pix_stop);
        draw_main_menu();
        menu = MENU_MAIN;
        redraw_info();
    }
    
    pressed_key = key_check();
    
    if(pressed_key != last_key)
    {
        switch(menu)
        {
            case MENU_MAIN:
                draw_main_menu();
                switch(pressed_key)
                {
                    case K_CONTEXT0:
                        menu = MENU_AMPL;
                        break;
                    case K_CONTEXT1:
                        menu = MENU_FREQ;
                        break;
                    case K_CONTEXT2:
                        menu = MENU_CONF;
                        break;
                }
                break;
            case MENU_AMPL:
                last_menu = MENU_AMPL;
                draw_ampl_menu();
                switch(pressed_key)
                {
                    case K_CONTEXT0:
                        sel_num_enter = ENTER_ATTEN;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT1:
                        sel_num_enter = ENTER_LVL;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT2:
                        sel_num_enter = ENTER_SCALE;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT3:
                        menu = MENU_MAIN;
                        break;
                }
                break;
            case MENU_FREQ:
                last_menu = MENU_FREQ;
                draw_freq_menu();
                switch(pressed_key)
                {
                    case K_CONTEXT0:
                        sel_num_enter = ENTER_START;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT1:
                        sel_num_enter = ENTER_STOP;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT2:
                        sel_num_enter = ENTER_CENTER;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT3:
                        sel_num_enter = ENTER_SPAN;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT4:
                        change_rbw();
                        break;
                    case K_CONTEXT5:
                        menu = MENU_MAIN;
                        break;
                }
                break;
            case MENU_CONF:
                last_menu = MENU_CONF;
                draw_conf_menu();
                switch(pressed_key)
                {
                    case K_CONTEXT0:
                        lvl_lo1_cal();
                        break;
                    case K_CONTEXT1:
                        menu = MENU_CAL_G;
                        break;
                    case K_CONTEXT2:
                        sel_num_enter = ENTER_F_CAL;
                        menu = MENU_ENTER_NUM;
                        break;
                    case K_CONTEXT3:
                        save_cal();
                        break;
                    case K_CONTEXT4:
                        menu = MENU_MAIN;
                        break;
                }
                break;
            case MENU_CAL_G:
                last_menu = MENU_CAL_G;
                draw_cal_gain_menu();
                switch(pressed_key)
                {
                    case K_CONTEXT0:
                        gain_corr_clr();    //after clearing calibration frequency is set to the first calibration point at 30MHz
                        meas_config.f_start = 30000000 - 10000000;
                        meas_config.f_stop = 30000000 + 10000000;
                        g_meas_stop = 1;
                        break;
                    case K_CONTEXT1:
                        g_meas_stop = 1;
                        g_cal_pt = 1;   //call for calibration - this makes sense only when appropriate calibration signal is provided (level CAL_LVL_DBM , frequency as the set center frequency which is n*30MHz)
                        break;
                    case K_CONTEXT2:
                        meas_config.f_start += 30000000;    //change frequency to calibrate at next frequency
                        meas_config.f_stop += 30000000;
                        g_meas_stop = 1;
                        break;
                    case K_CONTEXT3:
                        meas_config.f_start += 300000000;    //change frequency 10x forward
                        meas_config.f_stop += 300000000;
                        g_meas_stop = 1;
                        break;
                    case K_CONTEXT4:
                        gain_corr_apply();  //interpolate to fill missing calibration points
                        break;
                    case K_CONTEXT5:
                        menu = MENU_CONF;
                        break;
                }
                break;
            case MENU_ENTER_NUM:
                if(last_key >= K_CONTEXT5 && last_key <= K_CONTEXT0)     //TODO - clear number when entering this mode (do it better way??)
                {
                    i_numb_enter = 1;   //start writing number to buffer on [1], [0] is for minus
                    lock_dot = 0;
                    draw_units(sel_num_enter);
                }
                if(pressed_key == K_NUM0 || pressed_key == K_NUM1 || pressed_key == K_NUM2 || pressed_key == K_NUM3 ||
                   pressed_key == K_NUM4 || pressed_key == K_NUM5 || pressed_key == K_NUM6 || pressed_key == K_NUM7 ||
                   pressed_key == K_NUM8 || pressed_key == K_NUM9)
                {
                    if     (pressed_key == K_NUM0) numb_enter_str[i_numb_enter] = '0';
                    else if(pressed_key == K_NUM1) numb_enter_str[i_numb_enter] = '1';
                    else if(pressed_key == K_NUM2) numb_enter_str[i_numb_enter] = '2';
                    else if(pressed_key == K_NUM3) numb_enter_str[i_numb_enter] = '3';
                    else if(pressed_key == K_NUM4) numb_enter_str[i_numb_enter] = '4';
                    else if(pressed_key == K_NUM5) numb_enter_str[i_numb_enter] = '5';
                    else if(pressed_key == K_NUM6) numb_enter_str[i_numb_enter] = '6';
                    else if(pressed_key == K_NUM7) numb_enter_str[i_numb_enter] = '7';
                    else if(pressed_key == K_NUM8) numb_enter_str[i_numb_enter] = '8';
                    else if(pressed_key == K_NUM9) numb_enter_str[i_numb_enter] = '9';
                    pix_enter_num.x = enter_num_cursor;
                    draw_char(numb_enter_str[i_numb_enter], pix_enter_num);
                    enter_num_cursor += (CHAR_W + 1);
                    ++i_numb_enter;
                }
                else if(pressed_key == K_DOT && lock_dot == 0)
                {
                    numb_enter_str[i_numb_enter] = '.';
                    pix_enter_num.x = enter_num_cursor;
                    draw_char(numb_enter_str[i_numb_enter], pix_enter_num);
                    enter_num_cursor += (CHAR_W + 1);
                    ++i_numb_enter;
                    lock_dot = 1;
                }
                else if(pressed_key == K_MINUS)
                {
                    if(numb_enter_str[0] == '-')
                        numb_enter_str[0] = ' ';
                    else
                        numb_enter_str[0] = '-';
                    pix_enter_num.x = NUM_ENTER_FIELD_X;
                    draw_char(numb_enter_str[0], pix_enter_num);
                }
                else if(pressed_key == K_CLR)
                {
                    if(i_numb_enter > 1)
                    {
                        --i_numb_enter;
                        if(numb_enter_str[i_numb_enter] == '.')
                            lock_dot = 0;
                        numb_enter_str[i_numb_enter] = '\0';
                        enter_num_cursor -= (CHAR_W + 1);
                        pix_enter_num.x = enter_num_cursor;
                        draw_char(' ', pix_enter_num);
                    }
                }
                else if(pressed_key == K_ENTER || (pressed_key >= K_CONTEXT5 && pressed_key <= K_CONTEXT0))
                {
                    enter_num = 0;
                    numb_enter_str[i_numb_enter] = '\0';
                    dot_pos = -1;
                    for(i_numb_conv = 1; numb_enter_str[i_numb_conv] != '\0'; ++i_numb_conv)
                    {
                        if(numb_enter_str[i_numb_conv] == '.')
                            dot_pos = i_numb_conv;
                        if(numb_enter_str[i_numb_conv] >= '0' && numb_enter_str[i_numb_conv] <= '9')
                            if(dot_pos < 0)
                                enter_num = enter_num * 10 + (double)(numb_enter_str[i_numb_conv] - '0');
                            else
                                enter_num = enter_num + (double)(numb_enter_str[i_numb_conv] - '0') * pow(10, dot_pos - i_numb_conv);
                    }
                    if(numb_enter_str[0] == '-')
                        enter_num = 0 - enter_num;
                    menu = last_menu;
                    
                    //when number is entered an unit is selected by context menu buttons (e.g. Hz, kHz... if frequency value is entered)
                    //TODO - do it per menu item instead of selecting units based on the whole menu
                    if(last_menu == MENU_AMPL)
                        if(pressed_key == K_CONTEXT0 || pressed_key == K_ENTER) ;
                        else break;
                    if(last_menu == MENU_FREQ || last_menu == MENU_CONF)
                        if(pressed_key == K_CONTEXT0) enter_num *= 1000000000;
                        else if(pressed_key == K_CONTEXT1) enter_num *= 1000000;
                        else if(pressed_key == K_CONTEXT2) enter_num *= 1000;
                        else if(pressed_key == K_CONTEXT3 || pressed_key == K_ENTER) ;
                        else break;
                    enter_num_cursor = NUM_ENTER_FIELD_X + (CHAR_W + 1);
                    number_ready = 1;   //the number will be processed and put into appropriate place
                }
        }
        if(number_ready)
        {
            if     (sel_num_enter == ENTER_ATTEN) meas_config.atten = (int)enter_num;
            else if(sel_num_enter == ENTER_LVL)   meas_config.ref_lvl = enter_num;
            else if(sel_num_enter == ENTER_SCALE) meas_config.scale = enter_num;
            else if(sel_num_enter == ENTER_START) meas_config.f_start = enter_num;
            else if(sel_num_enter == ENTER_STOP)  meas_config.f_stop = enter_num;
            else if(sel_num_enter == ENTER_CENTER)  {   meas_config.f_start = meas_config.f_start - ((meas_config.f_stop + meas_config.f_start) / 2 - enter_num);
                                                        meas_config.f_stop  = meas_config.f_start + (enter_num - meas_config.f_start) * 2;}
            else if(sel_num_enter == ENTER_SPAN)    {   meas_config.f_start = (meas_config.f_stop + meas_config.f_start) / 2 - enter_num / 2;
                                                        meas_config.f_stop  = meas_config.f_start + enter_num;}
            else if(sel_num_enter == ENTER_F_CAL)  cal.f_cal = enter_num;

            g_meas_stop = 1;    //after changing something stop the sweep to avoid errors
            number_ready = 0;
            
            redraw_info();
        }
    }
    
    last_key = pressed_key;
    
    IFS0bits.T1IF = 0;  // Clear interrupt flag for timer 1
}

int key_check(void)
{
    int i_kbr;
    int key_num;
    int key_num_pressed;
    static int last_key_num;
    int mult_keys_err;
    static int key_debounce;
    
    key_num = -1;
    mult_keys_err = 0;
    PIN_KEYB_CLK = 0;

    for(i_kbr = 0; i_kbr < 24; ++i_kbr)
    {
        if(i_kbr % 8 == 0)
        {
            switch(i_kbr)
            {
                case 0:
                    PIN_KEYB_SEL0 = 0;
                    PIN_KEYB_SEL1 = 0;
                    break;
                case 8:
                    PIN_KEYB_SEL0 = 1;
                    PIN_KEYB_SEL1 = 0;
                    break;
                case 16:
                    PIN_KEYB_SEL0 = 0;
                    PIN_KEYB_SEL1 = 1;
                    break;
            }
            for(int32_t i_delay = 3; --i_delay; i_delay > 0);
            PIN_KEYB_LATCH = 0;
            PIN_KEYB_LATCH = 1;
        }
        
        if(PIN_KEYB_DATA == 1)
        {
            if(key_num == -1)
                key_num = i_kbr;
            else
                mult_keys_err = 1;
        }
        PIN_KEYB_CLK = 1;
        PIN_KEYB_CLK = 0;
    }
    
    if(mult_keys_err)
        key_num = -1;
    
    if(key_num > -1 && !mult_keys_err)
        if(last_key_num == key_num)
            ++key_debounce;
        else
            key_debounce = 0;
    last_key_num = key_num;
    
    if(key_debounce > 5)
        key_num_pressed = key_num;
    else
        key_num_pressed = -1;

    return key_num_pressed;
}

void draw_main_menu()
{
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
    button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "AMPL");
    button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "FREQ");
    button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CONF");
}

void draw_ampl_menu()
{
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
    button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "ATTEN");
    button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "REF");
    button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "SCALE");
    button_draw_clr(BUT_POS_3, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "EXIT");
}

void draw_freq_menu()
{
    char text_rbw[4][8] = {"RBW 1M ", "RBW 0M1", "RBW 10k", "RBW 1k "};    
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
    button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "START");
    button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "STOP");
    button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CENTER");
    button_draw_clr(BUT_POS_3, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "SPAN");
    button_draw_clr(BUT_POS_4, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, text_rbw[meas_config.rbw_sel]);
    button_draw_clr(BUT_POS_5, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "EXIT");
}

void draw_conf_menu()
{
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
    button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "LO1_LVL");
    button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CAL_GAIN");
    button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CAL_FREQ");
    button_draw_clr(BUT_POS_3, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "SAVE_CAL");
    button_draw_clr(BUT_POS_4, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "EXIT");
}

void draw_cal_gain_menu()
{
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
    button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CAL CLR");
    button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CAL POINT");
    button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "NEXT x1");
    button_draw_clr(BUT_POS_3, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "NEXT x10");
    button_draw_clr(BUT_POS_4, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "CAL APPLY");
    button_draw_clr(BUT_POS_5, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "EXIT");
}

void draw_units(int sel_num_enter)
{
    button_draw_clr(BUT_POS_CLR, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "");
	if(sel_num_enter == ENTER_START || sel_num_enter == ENTER_STOP || sel_num_enter == ENTER_CENTER || sel_num_enter == ENTER_SPAN || sel_num_enter == ENTER_F_CAL)
	{
        button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "GHz");
        button_draw_clr(BUT_POS_1, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "MHz");
        button_draw_clr(BUT_POS_2, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "kHz");
        button_draw_clr(BUT_POS_3, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "Hz");
	}
    else if(sel_num_enter == ENTER_ATTEN || sel_num_enter == ENTER_LVL || sel_num_enter == ENTER_SCALE)
	{
        button_draw_clr(BUT_POS_0, COLOR_BLACK, COLOR_YELLOW1, COLOR_RED1, "dB");
	}	
}


void change_rbw()
{
    g_meas_stop = 1;
	if(meas_config.rbw_sel < 3)
		++meas_config.rbw_sel;
	else
		meas_config.rbw_sel = 0;	
	draw_freq_menu();
}


void button_draw_clr(int button_position, int button_color_code, int bg_color_code, int text_color_code, char *button_text)
{
    struct pix_struct pix_start;
    struct pix_struct pix_stop;
    struct pix_struct pix_text;
	
	if(button_position == BUT_POS_CLR)
	{
		pix_stop.x = 140 + (pix_start.x = 660);
		pix_stop.y = SCREEN_H + (pix_start.y = 0);
		pix_start.r = (bg_color_code >> 6) & 0x7;
		pix_start.g = (bg_color_code >> 3) & 0x7;
		pix_start.b = (bg_color_code >> 0) & 0x7;
		draw_rect(pix_start, pix_stop); //menu background
	}
	else
	{
		pix_stop.x = 100 + (pix_start.x = 700);
		pix_stop.y = 30 + (pix_start.y = BUTT_START - button_position * BUTT_PITCH);
		pix_start.r = (button_color_code >> 6) & 0x7;
		pix_start.g = (button_color_code >> 3) & 0x7;
		pix_start.b = (button_color_code >> 0) & 0x7;
		draw_rect(pix_start, pix_stop); //button
		pix_text.x = pix_start.x + 3;
		pix_text.y = pix_start.y + 3;
		pix_text.r = (text_color_code >> 6) & 0x7;
		pix_text.g = (text_color_code >> 3) & 0x7;
		pix_text.b = (text_color_code >> 0) & 0x7;
		draw_text(button_text, pix_text); //button text
	}
}


void redraw_info(void)
{
    static struct pix_struct pix_enter_num = {NUM_ENTER_FIELD_X, 440, 0x3, 0x0, 0x0};
    struct pix_struct pix_start, pix_stop;
    char txt_fcen[] = "fcen=";
    char txt_fspan[] = "fspan=";
    char txt_ref[] = "ref=";
    char txt_atten[] = "atten=";
    char txt_fstart[] = "fstart=";
    char txt_fstop[] = "fstop=";
    
    pix_start.x = 0;
    pix_stop.x = PLOT_START_X + PLOT_W;
    pix_start.y = PLOT_STOP_Y + 1;
    pix_stop.y = SCREEN_H;
    pix_start.r = pix_start.g = pix_start.b = 0x0;
    draw_rect(pix_start, pix_stop);
    pix_start.y = 0;
    pix_stop.y = PLOT_START_Y - 1;
    draw_rect(pix_start, pix_stop);    //clean text fields    
    
    pix_start.x = 400;
    pix_start.y = PLOT_STOP_Y + 3;
    pix_start.r = pix_start.g = pix_start.b = 0x002;
    draw_text(txt_ref, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float(meas_config.ref_lvl, 0, pix_start);
    
    pix_start.x = 400;
    pix_start.y = PLOT_STOP_Y + CHAR_H + 5;
    draw_text(txt_atten, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float(((float)meas_config.atten), 0, pix_start);
    
    pix_start.x = 3;
    pix_start.y = CHAR_H + 2;
    draw_text(txt_fcen, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float((meas_config.f_start + meas_config.f_stop) / 2, 0, pix_start);
    
    pix_start.x = 3;
    pix_start.y = 1;
    draw_text(txt_fstart, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float(meas_config.f_start, 0, pix_start);
    
    pix_start.x = 400;
    pix_start.y = CHAR_H + 2;
    draw_text(txt_fstop, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float(meas_config.f_stop, 0, pix_start);
    
    pix_start.x = 400;
    pix_start.y = 1;
    draw_text(txt_fspan, pix_start);
    pix_start.x += CHAR_W * 8;
    print_float((meas_config.f_stop - meas_config.f_start), 0, pix_start);
}